#include <upc.h>
#define N   8 /* Dimension of vectors/matrices */

shared [N] int *a;
shared int *b;
shared int *c;

void matvec(shared [N] int A[], shared int B[], shared int C[]) {
	int c[N];
    upc_forall(int i = 0; i < N; i++; &C[i]) {
       //printf("Compute iteration %d by thread %d\n", i, MYTHREAD);

       //Using local var
       c[i] = 0;
       for(int j = 0; j < N; j++)
           c[i] += A[N*i+j] * B[j];

       /* 
       // Using "naive" add
       C[i] = 0;
       for(int j = 0; j < N; j++)
           C[i] += A[i][j] * B[j];
        */
    }

    upc_forall(int i = 0; i < N; i++; &C[i]) {
		C[i] = c[i];
	}
}

void printvec(shared int A[]) {
    printf("( ");
    for(int i = 0; i < N; i++) {
        printf("%2d ", A[i]);
    }
    printf(")");
}

int main(int argc, char* argv[])
{
   srand(MYTHREAD * 8);
   a = (shared [N] int*) upc_all_alloc(N, N*sizeof(int));
   b = (shared int*) upc_all_alloc(N, sizeof(int));
   c = (shared int*) upc_all_alloc(N, sizeof(int));
   upc_forall(int i = 0; i < N; i++; &a[N*i]) {
       for(int j = 0; j < N; j++) {
            a[N*i+j] = rand() % 10;
       }
   }
   upc_forall(int i = 0; i < N; i++; &b[i]) {
       //printf("Init iteration %d by thread %d\n", i, MYTHREAD);
       b[i] = rand() % 10;
       c[i] = 0;
   }

   upc_barrier;

   if (MYTHREAD == 0) {
       for(int x = 0; x < N; x++) {
            for(int y = 0; y < N; y++) {
                printf("%2d ", a[x*N+y]);
            }
           printf("\n");
       }

       printf(" . ");
       printvec(b);
       printf("T");
   }
   
   upc_barrier;

   matvec(a, b, b);

   upc_barrier;

   if(MYTHREAD == 0) {
       printf(" = ");
       printvec(b);
       printf("T\n");
   }

}
